using System;
using System.Collections;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting;
using System.Security.Principal;


namespace RemotingExample.Security
{
	
	public class ClientChannelSink :  BaseChannelObjectWithProperties, IClientChannelSink
	{
		
		public ClientChannelSink(String url, IClientChannelSink nextSink): base()
		{
			_url = url;
			_nextSink = nextSink;
		}

		public ClientChannelSink(IChannelSender channel, String url, Object remoteChannelData,
			IClientChannelSink nextSink): base()
		{
			_nextSink = nextSink;
		}


		// IClientChannelSink
		public void ProcessMessage(IMessage msg,
			ITransportHeaders requestHeaders, Stream requestStream,
			out ITransportHeaders responseHeaders, out Stream responseStream)
		{
			
			
	
			// initialize output message headers
			responseHeaders = null;
			responseStream = null;

			ClientSecurityContext context =  ClientSecurityContext.Current;
			if (context != null)
			{
				//look at current context state 
				switch (context.AuthState)
				{
					case AuthenticationStates.Authenticated:
						//if user already authenticated on application server, add app server SID to request headers
					
						if ((context.SessionID == null)||(context.SessionID.Length == 0))
						{
							throw new System.Security.SecurityException("SessionID must be set!");
						}
						requestHeaders[ChannelSinkHeaders.SID_HEADER] = context.SessionID;

						break;
					default :
						//if user  Not authenticated on app server
						requestHeaders[ChannelSinkHeaders.USER_NAME_HEADER] = context.Login;
						requestHeaders[ChannelSinkHeaders.PASSWORD_HEADER] = context.Password;
						context.ClearPassword();
						break;
				}
			}		
			
			//Do request to application server
			_nextSink.ProcessMessage(msg, requestHeaders, requestStream,
				out responseHeaders, out responseStream);
			
			AuthenticationStates serverAuth = AuthenticationStates.NotAuthenticated;

			string serverAuthHeader = (string)responseHeaders[ChannelSinkHeaders.AUTH_STATE_HEADER];
			if (serverAuthHeader != null)
			{
				serverAuth = (AuthenticationStates)System.Enum.Parse(typeof(AuthenticationStates), serverAuthHeader,true);
			}
			
			switch (serverAuth)
			{
				case AuthenticationStates.Authenticated:
					
					//If server response Authenticated  and  ClientContext not authenticated build ClientContext
					if (context.AuthState != AuthenticationStates.Authenticated)
					{
						string roles = responseHeaders[ChannelSinkHeaders.ROLES_HEADER].ToString();
						string[] rolesArr = roles.Split(new char[]{','});
						IIdentity identity = new GenericIdentity(ClientSecurityContext.Current.Login);
						IPrincipal userPrincipal = new GenericPrincipal(identity,rolesArr);
						context.SetAuthState(AuthenticationStates.Authenticated);
						context.SetPrincipal(userPrincipal);
						context.SetSessionID(responseHeaders[ChannelSinkHeaders.SID_HEADER].ToString());
						//Add Clientontext to container
						SecurityContextContainer.GetInstance()[context.SessionID] = context;
					}	
					break;
			}

		} 


		
		public void AsyncProcessRequest(IClientChannelSinkStack sinkStack, IMessage msg,
			ITransportHeaders headers, Stream stream)
		{

			// never gets called, this sink is always first
			sinkStack.Push(this, null);
			_nextSink.AsyncProcessRequest(sinkStack, msg, headers, stream);
		} // AsyncProcessRequest


		public void AsyncProcessResponse(IClientResponseChannelSinkStack sinkStack, Object state,
			ITransportHeaders headers, Stream stream)
		{
			
	
			sinkStack.AsyncProcessResponse(headers, stream);
		} // AsyncProcessResponse


		public Stream GetRequestStream(IMessage msg, ITransportHeaders headers)
		{
			// we always want a stream to read from
			return null;
		}


		public IClientChannelSink NextChannelSink
		{
			get { return _nextSink; }
		}



		// data members
		private string _url = null;
		private IClientChannelSink _nextSink = null;
	}

}
