
using System;
using System.Collections;
using System.Configuration;
using System.Security.Principal;
using System.Runtime.Remoting;

namespace RemotingExample.Security
{
	/// <summary>
	/// Client security context contains users data on Web server 
	/// </summary>
	public sealed class ClientSecurityContext : ISecurityContext
	{
		private IPrincipal user;
		private Hashtable  items;
		private string sessionID;
		private DateTime lastActivity;
		private string password;
		private string login;
		private AuthenticationStates authState;
				
		public ClientSecurityContext(string login, string password)
		{
			this.password = password;
			this.login = login;
			Current = this;
			authState = AuthenticationStates.NotAuthenticated;
			items = new Hashtable();
		}
		
		internal string Password
		{
			get
			{
				return password;
			}
		}
		
		public string Login
		{
			get
			{
				return login;
			}
		}

				
		public AuthenticationStates AuthState
		{
			get
			{
				return authState;
			}
		}

		internal void SetAuthState(AuthenticationStates state)
		{
			authState = state;
		}
		
		internal void SetPrincipal(IPrincipal user)
		{
			this.user = user;
		}

		public static ClientSecurityContext Current
		{
			get 
			{
				
				ClientSecurityContext currentContext = (ClientSecurityContext)System.Runtime.Remoting.Messaging.CallContext.GetData("ClientSecurityContext");;
				if (currentContext != null)
				{
					currentContext.lastActivity = DateTime.Now;
				}
				return currentContext;
			}
			set
			{
				if (value != null)
				{
					value.lastActivity = DateTime.Now;
				}
				System.Runtime.Remoting.Messaging.CallContext.SetData("ClientSecurityContext", value);
			}
		}
		
		
		public DateTime LastActivity
		{
			get 
			{
				return this.lastActivity;
			}
		}

		public string SessionID
		{
			get 
			{
				return this.sessionID;
			}
		}

		
		internal void SetSessionID(string sessionID)
		{
			this.sessionID = sessionID;
		}

		public Hashtable Items
		{
			get
			{
				return this.items;
			}
		}

		public IPrincipal User
		{
			get 
			{
				return user;
			}
			
		}
		#region IDisposable Members

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			if (disposing)
			{
				ClearPassword();

			}
		}
		
		~ClientSecurityContext()
		{
			Dispose(false);
		}

		public void ClearPassword()
		{
			password = "";
		}

		#endregion
	}
}

