using System;
using System.Collections;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Security.Principal;
using System.Threading;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Messaging;


namespace RemotingExample.Security
{
	
	public class ServerChannelSink : BaseChannelObjectWithProperties, IServerChannelSink
	{    
		
		public ServerChannelSink(IServerChannelSink nextSink) : base()
		{
			_nextSink = nextSink;
			
		} 


		public ServerChannelSink(IChannelReceiver channel, 
			IServerChannelSink nextSink) : base()
		{
			_nextSink = nextSink;
		} 



	
		public ServerProcessing ProcessMessage(IServerChannelSinkStack sinkStack,
			IMessage requestMsg, ITransportHeaders requestHeaders, Stream requestStream,
			out IMessage responseMsg, out ITransportHeaders responseHeaders,
			out Stream responseStream)
		{
			
			responseHeaders = null;
			responseStream = null;
			responseMsg = null;
			
			//Get Session ID from request Headers
			string SID = (string)requestHeaders[ChannelSinkHeaders.SID_HEADER];
			ServerSecurityContext context = null;
			
			//if no SID recivied
			if (SID == null)
			{
				//try to get login and password from request headers
				string userName = (string)requestHeaders[ChannelSinkHeaders.USER_NAME_HEADER];
				string password = (string)requestHeaders[ChannelSinkHeaders.PASSWORD_HEADER];
				AuthenticationStates authResult =  AuthenticationStates.NotAuthenticated;
				
				//if password and login finded
				if ((userName != null) && (password != null))
				{
					//try to authenticate user
					string roles;
					//Call authentication handler 
					authResult = Authenticate(userName,password, out roles);

					
					switch (authResult)
					{
						case AuthenticationStates.Authenticated:
							context = new ServerSecurityContext(userName,roles);
							context.SetAuthState(AuthenticationStates.Authenticated);
							SecurityContextContainer.GetInstance()[context.SessionID] = context;
							break;
						default:
							throw new System.Security.SecurityException("Authentication failed");
					}
				}
			}
			else
			{
				//Try to get user's session by SID
				context = (ServerSecurityContext)SecurityContextContainer.GetInstance()[SID];
				if (context == null)
				{
					throw new System.Security.SecurityException("Authentication failed");
				}
				else
				{
					ServerSecurityContext.Current = context;
				}
			}
			
			
			System.Security.Principal.IPrincipal orginalPrincipal = Thread.CurrentPrincipal;
			
			if (ServerSecurityContext.Current != null)
			{
				Thread.CurrentPrincipal = ServerSecurityContext.Current.Principal;
			}

			sinkStack.Push(this, null);
			ServerProcessing processing;
			processing = _nextSink.ProcessMessage(sinkStack, requestMsg, requestHeaders, requestStream ,out responseMsg, out responseHeaders, out responseStream);
			// pop us off the stack
			sinkStack.Pop(this);			
			
			//Rollback thread ptincipal
			Thread.CurrentPrincipal = orginalPrincipal;
            			
			AuthenticationStates serverAuthState =  AuthenticationStates.NotAuthenticated;

			if (ServerSecurityContext.Current != null)
				serverAuthState = context.AuthState;
			
			responseHeaders = new TransportHeaders();
			
			switch (serverAuthState)
			{
				case AuthenticationStates.Authenticated:
					responseHeaders[ChannelSinkHeaders.AUTH_STATE_HEADER] = AuthenticationStates.Authenticated;
					responseHeaders[ChannelSinkHeaders.SID_HEADER] = ServerSecurityContext.Current.SessionID;
					responseHeaders[ChannelSinkHeaders.ROLES_HEADER] = ServerSecurityContext.Current.Roles;
					break;
				default :
					responseHeaders[ChannelSinkHeaders.AUTH_STATE_HEADER] = serverAuthState;
				break;
			}
			
			//Clear current user security context.
			ServerSecurityContext.Current = null;

			// check for errors
			if (processing != ServerProcessing.Complete)
				throw new Exception("SecurityServerChannelSink: ProcessRequest: Did not get ServerProcessing.Complete from next message sink");
			return ServerProcessing.Complete;
		} // ProcessMessage

		/// <summary>
		/// Example authentication handler. 
		/// </summary>
		/// <param name="username"></param>
		/// <param name="password"></param>
		/// <param name="roles"></param>
		/// <returns></returns>
		private static AuthenticationStates Authenticate(string username,string password, out string roles)
		{
			roles  = "";
			
			System.Xml.XmlDocument passlist = new System.Xml.XmlDocument();
			passlist.Load("passwords.xml");
			
			string strXPath = String.Format( "user[@name='{0}']", username);
			System.Xml.XmlNode user=passlist.DocumentElement.SelectSingleNode( strXPath );    
			if (user==null)
				return AuthenticationStates.NotAuthenticated;
			
			string realyPassword  = user.Attributes["password"].Value;

			if (realyPassword!=password)
				return AuthenticationStates.NotAuthenticated;

			roles  = user.Attributes["roles"].Value; 
			return AuthenticationStates.Authenticated;
		}

				
		public void AsyncProcessResponse(IServerResponseChannelSinkStack sinkStack, Object state,
			IMessage msg, ITransportHeaders headers, Stream stream)
		{
			sinkStack.AsyncProcessResponse(msg, headers, stream);
		} // AsyncProcessResponse


		public Stream GetResponseStream(IServerResponseChannelSinkStack sinkStack, Object state,
			IMessage msg, ITransportHeaders headers)
		{
			// we always want a stream to read from 
			return null;
		} // GetResponseStream


		public IServerChannelSink NextChannelSink
		{
			get { return _nextSink; }
		}



		

		

		// data members
		private IServerChannelSink _nextSink = null;
		
		

	}

}
