using System;
using System.Collections;
using System.Configuration;
using System.Security.Principal;
using System.Runtime.Remoting;

namespace RemotingExample.Security
{
	/// <summary>
	/// Server security context store information about current user.
	/// 
	/// </summary>
	public sealed class ServerSecurityContext : IDisposable, ISecurityContext
	{
		private IPrincipal user; //User Principal attached to Thread
		private Hashtable  items; //User's objects storage
		private string sessionID; 
		private DateTime lastActivity;
		private string login;
		private string roles;
		private AuthenticationStates authState;

				
		public ServerSecurityContext(string login, string roles)
		{
			this.login = login;
			items = new Hashtable();
			this.sessionID = Guid.NewGuid().ToString();
			this.Roles = roles;
			string[] rolesArr=roles.Split(',');
			IIdentity identity = new GenericIdentity(login);
			GenericPrincipal userPrincipal = new GenericPrincipal(identity,rolesArr);
			this.user= userPrincipal;
			Current = this;
			
		}
		
		public string Login
		{
			get
			{
				return login;
			}
		}

		

		public AuthenticationStates AuthState
		{
			get
			{
				return authState;
			}
		}

		internal void SetAuthState(AuthenticationStates state)
		{
			authState = state;
		}

		internal string Roles
		{
			get
			{
				return roles;
			}set
			{
				roles = value;
			}
		}

				
		public static ServerSecurityContext Current
		{
			get 
			{
				
				ServerSecurityContext currentContext = (ServerSecurityContext)System.Runtime.Remoting.Messaging.CallContext.GetData("SecurityServerContext");;
				if (currentContext != null)
				{
					currentContext.lastActivity = DateTime.Now;
				}
				return currentContext;
			}set
			{
				if (value != null)
				{
					 value.lastActivity = DateTime.Now;
				}
				System.Runtime.Remoting.Messaging.CallContext.SetData("SecurityServerContext", value);
			}
		}
		
		
		public DateTime LastActivity
		{
			get 
			{
				return this.lastActivity;
			}
		}

		public string SessionID
		{
			get 
			{
				return this.sessionID;
			}
		}

		public IPrincipal Principal
		{
			get 
			{
				return this.user;
			}
		}
		
		
		public Hashtable Items
		{
			get
			{
				return this.items;
			}
		}

			
		public void Logout()
		{
			SecurityContextContainer.GetInstance().Remove(this.SessionID);
			this.Dispose();
		}

		

		#region IDisposable Members

		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			if (disposing)
			{
				//Dispose resources
			}
		}
		
		~ServerSecurityContext()
		{
			Dispose(false);
		}


		#endregion
	}
}

